/*
 * Copyright (C) 2011 phk.at
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



package at.phk.launcher2;

import android.app.*;
import android.content.*;
import android.content.pm.*;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.*;
import android.widget.*;


import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Observable;



public class Home extends Activity implements View.OnClickListener , View.OnLongClickListener {
	static final int REMOVE=-100;
	static final int UP=-200;
	static final int DOWN=-300;
	static final int CANCEL=-400;
	static final int APPS=-500;
	static final int FAVS=-600;
	static final int ADD=-700;
	static final int UNINSTALL=-800;
	static final int SETTINGS=-900;
	static final int RECENT=-1000;

	boolean include_back_button=true;
	boolean include_all_button=true;
	boolean include_recentmode_button=false;

	boolean recentmode=false;


	boolean favmode=true;
	AlertDialog alert=null;
	Button shortcut_button=null;
    
	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);

		setDefaultKeyMode(DEFAULT_KEYS_SEARCH_LOCAL);
      requestWindowFeature(Window.FEATURE_NO_TITLE);



		//show_fav();
	}


/////////
// manage shortcuts
////////
	static int fav_count(Context c) {
		return c.getSharedPreferences("phk.at",0).getInt("fav_count",0);
	}

	static String fav_title(Context c,int i) {
		return c.getSharedPreferences("phk.at",0).getString("fav_title"+i,"");
	}
	static String fav_pkg(Context c,int i) {
		return c.getSharedPreferences("phk.at",0).getString("fav_pkg"+i,"");
	}
	static String fav_act(Context c,int i) {
		return c.getSharedPreferences("phk.at",0).getString("fav_act"+i,"");
	}

	int fav_count() {
		return getSharedPreferences("phk.at",0).getInt("fav_count",0);
	}

	String fav_title(int i) {
		return getSharedPreferences("phk.at",0).getString("fav_title"+i,"");
	}
	String fav_pkg(int i) {
		return getSharedPreferences("phk.at",0).getString("fav_pkg"+i,"");
	}
	String fav_act(int i) {
		return getSharedPreferences("phk.at",0).getString("fav_act"+i,"");
	}
	void set_fav(int i,String title,String pkg,String act) {
		getSharedPreferences("phk.at",0).edit()
			.putString("fav_title"+i,title)
			.putString("fav_pkg"+i,pkg)
			.putString("fav_act"+i,act)
			.commit();
	}

	void add_fav(String title,String pkg,String act) {
		int i=fav_count();
		set_fav(i,title,pkg,act);
		getSharedPreferences("phk.at",0).edit().putInt("fav_count",i+1).commit();
		Log.d("phkat","add_fav "+i);
	}
	void remove_fav(int id) {
		int c=fav_count();

		for (int i=id;i<c-1;i++) {
			set_fav(i,
				fav_title(i+1),
				fav_pkg(i+1),
				fav_act(i+1)
			);
		}
		getSharedPreferences("phk.at",0).edit().putInt("fav_count",c-1).commit();
		Log.d("phkat","remove_fav "+id+"  "+c);
	}
	void move_fav(int i,int dir) {
		int c=fav_count();
		if (i<0) return;
		if (i>=c) return;
		if (dir==0) return;
		if (dir<0) {
			if (i<1) return;
			String title=fav_title(i);
			String pkg=fav_pkg(i);
			String act=fav_act(i);

			set_fav(i,
				fav_title(i-1),
				fav_pkg(i-1),
				fav_act(i-1)
			);
			
			set_fav(i-1,
				title,
				pkg,
				act
			);
			return;
		}
		if (i>=c-1) return;
		String title=fav_title(i);
		String pkg=fav_pkg(i);
		String act=fav_act(i);

		set_fav(i,
			fav_title(i+1),
			fav_pkg(i+1),
			fav_act(i+1)
		);
		
		set_fav(i+1,
			title,
			pkg,
			act
		);
		return;
	}



/////////
// show lists
////////

	protected void show_fav() {
		favmode=true;
		LinearLayout ll=new LinearLayout(this);
		LinearLayout.LayoutParams lp=new LinearLayout.LayoutParams(
		ViewGroup.LayoutParams.FILL_PARENT,ViewGroup.LayoutParams.WRAP_CONTENT
		);
		lp.weight=0.1f;
		ll.setLayoutParams(lp);
		ll.setOrientation(LinearLayout.VERTICAL);

		int count=fav_count();
		for (int i=0;i<count;i++) {
			add_button(ll,i,fav_title(i),new String[]{
				fav_pkg(i),
				fav_act(i)
			});
		}
		if (include_all_button) {
			Button b=new Button(this);
			b.setText("all");
			b.setId(APPS);
			b.setOnClickListener(this);
			ll.addView(b);
		}




		ScrollView sv=new ScrollView(this);
		sv.addView(ll);
		setContentView(sv);
	}






	static class InstallTimeComparator implements Comparator<ResolveInfo> {

		PackageManager pm=null;

		static long get_install_time(PackageManager pm,String pkgname) {
			try {
				ApplicationInfo appInfo = pm.getApplicationInfo(pkgname, 0);
				String appFile = appInfo.sourceDir;
				long installed = new File(appFile).lastModified();
				return installed;
			} catch (Exception e) { return 0; }
		}


		public final int compare(ResolveInfo a,ResolveInfo b) {
			long la=get_install_time( pm, a.activityInfo.applicationInfo.packageName);
			long lb=get_install_time( pm, b.activityInfo.applicationInfo.packageName);
			if (la<lb) { return 1; }
			if (la==lb) return 0;
			return -1;
		}

		static InstallTimeComparator instance=new InstallTimeComparator();

	}


	protected void show_apps() {
		favmode=false;
		LinearLayout ll=new LinearLayout(this);
		LinearLayout.LayoutParams lp=new LinearLayout.LayoutParams(
		ViewGroup.LayoutParams.FILL_PARENT,ViewGroup.LayoutParams.WRAP_CONTENT
		);
		lp.weight=0.1f;
		ll.setLayoutParams(lp);
		ll.setOrientation(LinearLayout.VERTICAL);

		Button b=null;
		if (include_back_button) {
			b=new Button(this);
			b.setText("back");
			b.setId(FAVS);
			b.setOnClickListener(this);
			ll.addView(b);
		}
		if (include_recentmode_button) {
			b=new Button(this);
			if (recentmode) {
				b.setText("Sort: time");
			} else {
				b.setText("Sort: name");
			}
			b.setId(RECENT);
			b.setOnClickListener(this);
			ll.addView(b);
		}

		PackageManager manager = getPackageManager();
		Intent mainIntent = new Intent(Intent.ACTION_MAIN, null);
		mainIntent.addCategory(Intent.CATEGORY_LAUNCHER);
		final List<ResolveInfo> apps = manager.queryIntentActivities(mainIntent, 0);

		if (recentmode) {
			InstallTimeComparator.instance.pm=manager;
			Collections.sort(apps, InstallTimeComparator.instance);
		} else {
			Collections.sort(apps, new ResolveInfo.DisplayNameComparator(manager));
		}

		if (apps != null) { 
			final int count = apps.size();
			for (int i = 0; i < count; i++) {
				ResolveInfo info = apps.get(i);
				add_button(
					ll,
					i,
					info.loadLabel(manager),
					new String[]{
						info.activityInfo.applicationInfo.packageName,
						info.activityInfo.name
					}
				);
			}
		}
		ScrollView sv=new ScrollView(this);
		sv.addView(ll);
		setContentView(sv);
	}

	public void add_button(LinearLayout ll,int i,CharSequence title,Object data) {
		Button b=new Button(this);
		b.setId(i);
		b.setText(title);
		b.setTag(data);
		b.setOnClickListener(this);
		b.setOnLongClickListener(this);
		ll.addView(b);
	}

	public void add_button(LinearLayout ll,int i,String title,Object data) {
		Button b=new Button(this);
		b.setId(i);
		b.setText(title);
		b.setTag(data);
		b.setOnClickListener(this);
		b.setOnLongClickListener(this);
		ll.addView(b);
	}

   public void toast(String q) {
      Toast.makeText(this,q, Toast.LENGTH_SHORT).show();
   }

/////////
// handle events
////////


	public boolean onLongClick(View v) {
		Log.i("phkat", "long buttonclick "+v.getTag());
		if (favmode) {
			String[] tag=(String[])v.getTag();
			String title=((Button)v).getText().toString();

			LinearLayout ll=new LinearLayout(this);
			LinearLayout.LayoutParams lp=new LinearLayout.LayoutParams(
			ViewGroup.LayoutParams.FILL_PARENT,ViewGroup.LayoutParams.WRAP_CONTENT
			);
			lp.weight=0.1f;
			ll.setLayoutParams(lp);
			ll.setOrientation(LinearLayout.VERTICAL);

			Button b=new Button(this);
			b.setText("Remove shortcut");
			b.setId(REMOVE);
			b.setOnClickListener(this);
			ll.addView(b);
			b=new Button(this);
			b.setText("Move up");
			b.setOnClickListener(this);
			b.setId(UP);
			ll.addView(b);
			b=new Button(this);
			b.setText("Move down");
			b.setOnClickListener(this);
			b.setId(DOWN);
			ll.addView(b);
			ScrollView sv=new ScrollView(this);
			sv.addView(ll);
		
			shortcut_button=((Button)v);
			alert=(new AlertDialog.Builder(this).setView(sv).setTitle(title).setNegativeButton("Cancel",null).show());

		} else {
			String title=((Button)v).getText().toString();
			LinearLayout ll=new LinearLayout(this);
			LinearLayout.LayoutParams lp=new LinearLayout.LayoutParams(
			ViewGroup.LayoutParams.FILL_PARENT,ViewGroup.LayoutParams.WRAP_CONTENT
			);
			lp.weight=0.1f;
			ll.setLayoutParams(lp);
			ll.setOrientation(LinearLayout.VERTICAL);

			Button b=new Button(this);
			b.setText("Add shortcut");
			b.setId(ADD);
			b.setOnClickListener(this);
			ll.addView(b);
			/*b=new Button(this);
			b.setText("app settings");
			b.setId(SETTINGS);
			b.setOnClickListener(this);
			ll.addView(b);*/
			b=new Button(this);
			b.setText("Uninstall app");
			b.setId(UNINSTALL);
			b.setOnClickListener(this);
			ll.addView(b);

			ScrollView sv=new ScrollView(this);
			sv.addView(ll);

			shortcut_button=((Button)v);

			alert=(new AlertDialog.Builder(this).setView(sv).setTitle(title).setNegativeButton("Cancel",null).show());

		}
		return true;
	}


	public void update() {
		if (favmode) {
			show_fav();
		} else {
			show_apps();
		}
	}

	public void switch_update() {
		favmode=!favmode;
		update();
	}


	protected void onActivityResult(int requestCode,
															int resultCode,
															String strdata,
															Bundle bundle)
	{
		Log.d("phk.at","onActivityResult "+requestCode+" "+resultCode);
		update();
	}



	public void onClick(View v) {
		if (v==null) { return; }
		Log.i("phkat", "buttonclick "+v.getTag()+"  "+v.getId());

		switch (v.getId()) {
			case RECENT:
				recentmode=!recentmode;
				//show_apps();
				update();
				alert=null;
				shortcut_button=null;
				return;
			case APPS:
				//show_apps();
				switch_update();
				alert=null;
				shortcut_button=null;
				return;
			case FAVS:
				//show_fav();
				switch_update();
				alert=null;
				shortcut_button=null;
				return;
			case UNINSTALL: {
				String[] tag=(String[])shortcut_button.getTag();
				Intent intent = new Intent(Intent.ACTION_DELETE);
				intent.setData(Uri.parse("package:"+tag[0]));
				//startActivity(intent);
				//startSubActivity(intent, 111);
				startActivityForResult(intent,111);
				if (alert!=null) alert.dismiss();
				alert=null;
				shortcut_button=null;
				return; }
			/*case SETTINGS: {
				Intent intent = new Intent();
				intent.setAction(android.provider.Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
				//Constant Value: "android.settings.APPLICATION_DETAILS_SETTINGS" 
				//requires api lvl 9
				String[] tag=(String[])shortcut_button.getTag();
				intent.setData(Uri.parse("package:"+tag[0]));
				startActivity(intent); 
				return; }*/
			case ADD: {
				String title=shortcut_button.getText().toString();
				String[] tag=(String[])shortcut_button.getTag();
				Log.i("phkat", "long apps "+title+" "+tag[0]+" "+tag[1]);
				add_fav(
					title,
					tag[0],
					tag[1]
				);
				toast("added shortcut: "+title);
				//show_fav();
				switch_update();
				if (alert!=null) alert.dismiss();
				alert=null;
				shortcut_button=null;
				return; }
			case REMOVE: {
				String title=shortcut_button.getText().toString();
				remove_fav(shortcut_button.getId());
				update();
				if (alert!=null) alert.dismiss();
				alert=null;
				shortcut_button=null;
				toast("removed "+title);
				return; }
			case DOWN: {
				String title=shortcut_button.getText().toString();
				move_fav(shortcut_button.getId(),1);
				update();
				if (alert!=null) alert.dismiss();
				alert=null;
				shortcut_button=null;
				toast("moved "+title);
				return; }
			case UP: {
				String title=shortcut_button.getText().toString();
				move_fav(shortcut_button.getId(),-1);
				update();
				if (alert!=null) alert.dismiss();
				alert=null;
				shortcut_button=null;
				toast("moved "+title);
				return; }
			default: break;
		}

		try {
			String[] tag=(String[])v.getTag();
			startActivity(get_start_intent(tag[0],tag[1]));
		} catch (Exception e) {
			toast("error");
		}
		alert=null;
		shortcut_button=null;
	}
		


	static Intent get_start_intent(String pkg,String act) {
		Intent i=new Intent(Intent.ACTION_MAIN);
		i.addCategory(Intent.CATEGORY_LAUNCHER);
		i.setComponent(new ComponentName(pkg,act));
		i.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK| Intent.FLAG_ACTIVITY_RESET_TASK_IF_NEEDED);
		return i;
	}



}
